import os
import shutil
import unittest
from cpenv.hooks import HookFinder
from cpenv.models import VirtualEnvironment
from nose.tools import raises
from . import data_path, make_files


HOOK_TEXT = '''
def run(env):
    return True
'''
MODULE_HOOK_TEXT = '''
def run(env, module):
    return True
'''


def setup_module():
    env_files = (
        data_path('home', 'testenv', 'environment.yml'),
        data_path('home', 'testenv', 'modules', 'testmod', 'module.yml'),
    )
    global_hook_files = (
        data_path('home', 'hooks', 'precreate.py'),
        data_path('home', 'hooks', 'postcreate.py'),
        data_path('home', 'hooks', 'preactivate.py'),
        data_path('home', 'hooks', 'postactivate.py'),
    )
    env_hook_files = (
        data_path('home', 'testenv', 'hooks', 'preactivate.py'),
        data_path('home', 'testenv', 'hooks', 'preactivatemodule.py'),
    )
    mod_hook_files = (
        data_path('home', 'testenv', 'modules',
                  'testmod', 'hooks', 'postactivatemodule.py'),
    )
    make_files(text='', *env_files)
    make_files(text=HOOK_TEXT, *global_hook_files)
    make_files(text=HOOK_TEXT, *env_hook_files)
    make_files(text=MODULE_HOOK_TEXT, *mod_hook_files)
    os.environ['CPENV_HOME'] = data_path('home')


def teardown_module():
    shutil.rmtree(data_path('home'))


class TestHookFinder(unittest.TestCase):

    def setUp(self):

        self.hook_finder = HookFinder(
            data_path('home', 'testenv', 'modules', 'testmod', 'hooks'),
            data_path('home', 'testenv', 'hooks'),
            data_path('home', 'hooks'),
        )

    def test_hook_resolution(self):
        '''Test hook resolution'''

        hook = self.hook_finder('precreate')
        assert hook.__file__ == data_path('home', 'hooks', 'precreate.py')

        hook = self.hook_finder('preactivate')
        assert hook.__file__ == data_path('home', 'testenv',
                                          'hooks', 'preactivate.py')

        hook = self.hook_finder('postactivatemodule')
        assert hook.__file__ == data_path('home', 'testenv', 'modules',
                                          'testmod', 'hooks',
                                          'postactivatemodule.py')

    def test_env_hook(self):
        '''Test run environment hook'''

        env = VirtualEnvironment(data_path('home', 'testenv'))
        hook = self.hook_finder('precreate')
        assert hook.run(env) == True

    def test_mod_hook(self):
        '''Test run module hook'''

        env = VirtualEnvironment(data_path('home', 'testenv'))
        mod = env.get_module('testmod')
        hook = self.hook_finder('postactivatemodule')
        assert hook.run(env, mod) == True

    def test_hook_not_found(self):
        '''Test hook not found'''

        hook = self.hook_finder('ppp')
        assert hook is None

    @raises(SyntaxError)
    def test_hook_syntaxerror(self):
        '''Test hook compile raises syntaxerror'''

        make_files(data_path('home', 'hooks', 'badhook.py'), text='x is = 4')
        self.hook_finder('badhook')

    @raises(IndexError)
    def test_hook_runerror(self):
        '''Test hook run raises error'''

        make_files(data_path('home', 'hooks', 'badrunhook.py'),
                   text='def run(env):\n\treturn env[1]')
        self.hook_finder('badrunhook').run([0])



import os
from contextlib import contextmanager
from functools import partial

data_path = partial(os.path.join, os.path.dirname(__file__), 'data')


@contextmanager
def cwd(new_cwd):

    old_cwd = os.getcwd()

    try:
        os.chdir(new_cwd)
        yield
    except:
        raise
    finally:
        os.chdir(old_cwd)


def touch(filepath):

    with open(filepath, 'a'):
        os.utime(filepath, None)


def make_files(*filepaths, **kwargs):
    data = kwargs.get('text', None)

    for filepath in filepaths:
        d = os.path.dirname(filepath)

        try:
            os.makedirs(d)
        except:
            pass

        if not data:
            touch(filepath)
        else:
            with open(filepath, 'w') as f:
                f.write(data)



import os
import sys


def main(*args):
    print('\n\nRunning Test Suite...\n\n')

    if 'cover' in args:
        cmd = ('nosetests -v --logging-clear-handlers '
               '--with-coverage --cover-package=cpenv')
    else:
        cmd = 'nosetests -v --logging-clear-handlers'

    os.system(cmd)


if __name__ == '__main__':
    main(*sys.argv)



import os
import shutil
import mock
import sys
from cpenv.resolver import Resolver
from cpenv.models import VirtualEnvironment, Module
from cpenv import platform
from nose.tools import raises
from . import data_path, make_files, cwd

ENV_TEXT = '''
environment:
    UNRESOLVED_PATH: $NOVAR
    RESOLVED_PATH: $ENVIRON/resolved
    PLATFORM_PATH:
        win: environ_win
        osx: environ_osx
        linux: environ_linux
    MULTI_PLATFORM_PATH:
        - nonplat
        - win:
            - $PYVER/wina
            - $PYVER/winb
          osx:
            - $PYVER/osxa
            - $PYVER/osxb
          linux:
            - $PYVER/linuxa
            - $PYVER/linuxb
'''


def setup_module():
    files = (
        data_path('home', 'testenv', 'environment.yml'),
        data_path('home', 'testenv', 'modules', 'testmod', 'module.yml'),
        data_path('not_home', 'testenv', 'environment.yml'),
        data_path('cached', 'cachedenv', 'environment.yml')
    )
    make_files(text=ENV_TEXT, *files)
    os.environ['CPENV_HOME'] = data_path('home')


def teardown_module():
    shutil.rmtree(data_path('home'))
    shutil.rmtree(data_path('not_home'))
    shutil.rmtree(data_path('cached'))


def test_resolve_home():
    '''Resolve environment in CPENV_HOME'''

    r = Resolver('testenv')
    r.resolve()

    assert r.resolved[0].path == data_path('home', 'testenv')


def test_resolve_relative():
    '''Resolve environment from relative path'''

    with cwd(data_path('not_home')):
        r = Resolver('testenv')
        r.resolve()

    assert r.resolved[0].path == data_path('not_home', 'testenv')


def test_resolve_absolute():
    '''Resolve environment from absolute path'''

    with cwd(data_path('not_home')):
        r = Resolver(data_path('home', 'testenv'))
        r.resolve()

    assert r.resolved[0].path == data_path('home', 'testenv')


def test_resolve_cache():
    '''Resolve environment from cache'''

    cached_env_path = data_path('cached', 'cachedenv')
    mock_cache = mock.Mock()
    mock_cache.find = mock.Mock(
        return_value=VirtualEnvironment(cached_env_path)
    )

    r = Resolver('cachedenv', cache=mock_cache)
    r.resolve()

    assert r.resolved[0].path == cached_env_path


def test_resolve_multi_args():
    '''Resolve multiple paths'''

    r = Resolver('testenv', 'testmod')
    r.resolve()

    assert isinstance(r.resolved[0], VirtualEnvironment)
    assert isinstance(r.resolved[1], Module)


def test_combine_multi_args():
    '''Resolve combine multiple paths'''

    pyver = str(sys.version[:3])
    expected = {
        'PATH': [data_path('home', 'testenv', 'bin')],
        'CPENV_ACTIVE_MODULES': [],
        'UNRESOLVED_PATH': '$NOVAR',
        'RESOLVED_PATH': data_path('home', 'testenv', 'resolved'),
        'PLATFORM_PATH': 'environ_' + platform,
        'MULTI_PLATFORM_PATH': [
            'nonplat',
            pyver + '/' + platform + 'a',
            pyver + '/' + platform + 'b',
        ]
    }

    r = Resolver('testenv', 'testmod')
    r.resolve()
    combined = r.combine()

    for k in expected.keys():
        assert expected[k] == combined[k]


@raises(NameError)
def test_nonexistant_virtualenv():
    '''Raise NameError when environment does not exist'''

    r = Resolver('does_not_exist')
    r.resolve()


@raises(NameError)
def test_nonexistant_module():
    '''Raise NameError when module does not exist'''

    r = Resolver('testenv', 'does_not_exist')
    r.resolve()


@raises(NameError)
def test_multi_module_does_not_exist():
    '''Raise NameError when a module does not exist'''

    r = Resolver('testenv', 'testmod', 'does_not_exist')
    r.resolve()
